# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>


__version__ = "0.3.0"


from .abstractions import SingletonMeta
from .connectors import BaseAsyncApiConnector
from .ecelery import create_celery_custom_task_class
from .econnections import RedisClient, new_umongo_instance
from .efastapi import create_fastapi_api_app, create_fastapi_server_app
from .ehttpx import monkey_patch_httpx
from .elogger import (
    CorrelationIdFilter,
    EnsureValidExtraFilter,
    RequestLoggingMiddleware,
    create_json_logger,
    get_correlation_id,
    set_correlation_id,
)
from .env_settings import (
    BaseSettingMixin,
    GunicornSettings,
    MongoDBSettings,
    OpenTelemetrySettings,
    RabbitMQSettings,
    RedisSettings,
    UvicornSettings,
)
from .open_telemetry import setup_open_telemetry

__all__ = [
    "SingletonMeta",
    "BaseAsyncApiConnector",
    "create_celery_custom_task_class",
    "create_fastapi_api_app",
    "create_fastapi_server_app",
    "monkey_patch_httpx",
    "create_json_logger",
    "get_correlation_id",
    "set_correlation_id",
    "RequestLoggingMiddleware",
    "CorrelationIdFilter",
    "EnsureValidExtraFilter",
    "BaseSettingMixin",
    "GunicornSettings",
    "MongoDBSettings",
    "RedisSettings",
    "OpenTelemetrySettings",
    "RabbitMQSettings",
    "UvicornSettings",
    "setup_open_telemetry",
    "new_umongo_instance",
    "RedisClient",
]



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import os

SELECTED_ENV_NAME = os.environ.get("APP_ENV_NAME")


class BaseSettingMixin:
    @classmethod
    def get_selected_env(cls) -> str:
        if SELECTED_ENV_NAME in ("dev", "stg", "prod", "local", "redirect"):
            env_name = f"{SELECTED_ENV_NAME}.env"
        else:
            env_name = ".env"
        return env_name

    @classmethod
    def get_env_file_path(cls, env_name: str) -> str:
        env_file_path = os.path.join(os.getcwd(), "envs", env_name)
        if not os.path.isfile(env_file_path):
            with open(env_file_path, "w") as f:  # noqa # NOSONAR
                pass
        return env_file_path

    def setup(self, from_env: str, setup_proxy: bool = True):
        if setup_proxy:
            self.add_custom_proxy()
        self.notify_environment(from_env)

    def notify_environment(self, from_env: str):
        print(2 * "\n" + 50 * "-" + "\n")
        print("you select environment: ", from_env)
        print(f'http_proxy {os.environ.get("http_proxy")}')
        print(f'https_proxy {os.environ.get("https_proxy")}')
        print(f'no_proxy {os.environ.get("no_proxy")}')
        print("\n" + 50 * "-" + 2 * "\n")

    def add_custom_proxy(self):
        if (
            not hasattr(self, "SERVICE_USE_PROXY")
            or not hasattr(self, "SERVICE_PROXY_ADDR")
            or not hasattr(self, "SERVICE_NO_PROXY")
        ):
            print("Not found proxy configurations")
            return

        if not getattr(self, "SERVICE_USE_PROXY"):
            return

        no_proxy = os.environ.get("no_proxy")

        if getattr(self, "SERVICE_USE_PROXY"):
            os.environ["http_proxy"] = getattr(self, "SERVICE_PROXY_ADDR")
            os.environ["https_proxy"] = getattr(self, "SERVICE_PROXY_ADDR")
        if getattr(self, "SERVICE_NO_PROXY"):
            if no_proxy:
                os.environ["no_proxy"] = (
                    no_proxy + "," + getattr(self, "SERVICE_NO_PROXY")
                )
            else:
                os.environ["no_proxy"] = getattr(self, "SERVICE_NO_PROXY")



# -*- coding: utf-8 -*-
from redis.asyncio import ConnectionPool, Redis

from ..abstractions import SingletonMeta


class RedisClient(metaclass=SingletonMeta):
    def __init__(self, redis_url: str, max_conns: int = 20) -> None:
        self.client = self._new_client(redis_url, max_conns)

    def _new_client(self, redis_url: str, max_conns: int) -> Redis:
        """
        Create a new Redis client.
        redis_url example: "redis://[:password]@host:port/db_number"
                            "redis://user:password@host:port/db_number"
        Args:
            redis_url (str): Redis connection URL.
            max_conns (int): Maximum number of connections in the pool.
        Returns:
            Redis: An instance of Redis client.
        """
        pool = ConnectionPool.from_url(
            url=redis_url,
            max_connections=max_conns,
            encoding="utf-8",
            decode_responses=True,
        )
        client = Redis(connection_pool=pool)
        return client



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from .base import BaseSettingMixin
from .gunicorn_settings import GunicornSettings
from .mongo_settings import MongoDBSettings
from .open_telemetry import OpenTelemetrySettings
from .rabbitmq_settings import RabbitMQSettings
from .redis_settings import RedisSettings
from .uvicorn_settings import UvicornSettings

__all__ = [
    "BaseSettingMixin",
    "GunicornSettings",
    "MongoDBSettings",
    "RedisSettings",
    "OpenTelemetrySettings",
    "RabbitMQSettings",
    "UvicornSettings",
]



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from typing import Optional

from pydantic_settings import BaseSettings


class GunicornSettings(BaseSettings):
    GUNICORN_HOST: Optional[str] = "0.0.0.0"
    GUNICORN_PORT: Optional[str] = "5002"
    GUNICORN_BIND_PATH: Optional[str] = None
    GUNICORN_WORKER_CONCURRENCY: Optional[str] = "2"
    GUNICORN_NUMBER_OF_WORKERS: Optional[str] = "2"



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>
from .custom_schedule import MyTaskiqScheduler
from .installler import new_taskiq_broker
from .middleware_logging import LoggingMiddlewareWorker
from .mongo_result_backend import MongoDBResultBackend

__all__ = [
    "new_taskiq_broker",
    "MongoDBResultBackend",
    "LoggingMiddlewareWorker",
    "MyTaskiqScheduler",
]



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from typing import Optional

from pydantic_settings import BaseSettings


class UvicornSettings(BaseSettings):
    UVICORN_HOST: Optional[str] = "0.0.0.0"
    UVICORN_PORT: Optional[str] = "5002"
    UVICORN_LOG_LEVEL: Optional[str] = "info"
    UVICORN_RELOAD: Optional[bool] = False
    UVICORN_BIND_PATH: Optional[str] = None
    UVICORN_WORKER_CONCURRENCY: Optional[str] = "2"
    UVICORN_NUMBER_OF_WORKERS: Optional[str] = "2"
    UVICORN_LOOP: Optional[str] = "uvloop"  # "auto", "asyncio", "uvloop"



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import threading
from contextvars import ContextVar

ctxv_correlation_id = ContextVar("correlation_id", default=None)
tlocal_correlation_id = threading.local()  # for non-async context, fallback


def set_correlation_id(correlation_id: str):
    """Set correlation ID for the current context (async or thread)."""
    try:
        ctxv_correlation_id.set(correlation_id)
    except Exception:
        pass
    tlocal_correlation_id.correlation_id = correlation_id


def get_correlation_id() -> str:
    """Get correlation ID for the current context (async or thread)."""
    try:
        cid = ctxv_correlation_id.get()
        if cid:
            return cid
    except LookupError:
        pass
    return getattr(tlocal_correlation_id, "correlation_id", None)



# -*- coding: utf-8 -*-
from datetime import datetime, timezone
from typing import Any, Optional

from motor.motor_asyncio import AsyncIOMotorClient
from taskiq import AsyncResultBackend, TaskiqResult


class MongoDBResultBackend(AsyncResultBackend):
    def __init__(self, mongo_url: str, database_name: str, collection_name: str):
        self.mongo_url = mongo_url
        self.database_name = database_name
        self.collection_name = collection_name
        self.client = None
        self.db = None
        self.collection = None

    async def startup(self) -> None:
        """Khởi tạo kết nối tới MongoDB."""
        self.client = AsyncIOMotorClient(self.mongo_url)
        self.db = self.client[self.database_name]
        self.collection = self.db[self.collection_name]
        return

    async def shutdown(self) -> None:
        """Đóng kết nối MongoDB."""
        if self.client:
            self.client.close()
        return

    async def set_result(self, task_id: str, result: TaskiqResult[Any]) -> None:
        """Lưu kết quả vào MongoDB."""
        # print('set_result', result)
        await self.collection.update_one(
            {"task_id": task_id},
            {
                "$set": {
                    "task_id": task_id,
                    "task_name": result.labels.get("task_name", "unset"),
                    "result": result.return_value,
                    "success": not result.is_err,
                    "error": str(result.error) if result.error else None,
                    "execution_time": (
                        result.execution_time
                        if result.execution_time is not None
                        else 0.0
                    ),
                    "labels": result.labels if result.labels is not None else {},
                    "updated_at": datetime.now(timezone.utc),
                }
            },
            upsert=True,
        )

    async def get_result(
        self, task_id: str, with_logs: bool = False
    ) -> Optional[TaskiqResult[Any]]:
        """Lấy kết quả từ MongoDB."""
        doc = await self.collection.find_one({"task_id": task_id})
        if doc:
            return TaskiqResult(
                return_value=doc.get("result"),
                is_err=not doc.get("success", True),
                error=doc.get("error"),
                log=doc.get("log"),
                execution_time=doc.get("execution_time", 0.0),
                labels=doc.get("labels", {}),
            )
        return None

    async def is_result_ready(self, task_id: str) -> bool:
        """Kiểm tra xem kết quả đã sẵn sàng chưa."""
        doc = await self.collection.find_one({"task_id": task_id})
        return doc is not None



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging

from celery import Task

from ..elogger import get_correlation_id, set_correlation_id


def create_celery_custom_task_class(logger: logging.Logger):
    class CeleryCustomTask(Task):
        def apply_async(
            self,
            args=None,
            kwargs=None,
            task_id=None,
            producer=None,
            link=None,
            link_error=None,
            shadow=None,
            **options,
        ):
            cid = get_correlation_id()
            # add log event id to header
            if cid:
                headers = options.get("headers", {})
                if "cid" not in headers:
                    headers["cid"] = cid
                    options["headers"] = headers

            # add task to queue
            task_result = super().apply_async(
                args, kwargs, task_id, producer, link, link_error, shadow, **options
            )
            logger.info(
                f"called task {self.name} with correlation_id: {cid}",
                extra={
                    "loc": "core0006",
                    "celery_task_id": task_result.id,
                    "celery_task_name": self.name,
                    "celery_task_args": str(args),
                    "celery_task_kwargs": str(kwargs),
                },
            )
            return task_result

        def before_start(self, task_id, args, kwargs):
            headers = self.request.headers
            if isinstance(headers, dict) and "cid" in headers:
                # get log event id from headers
                cid = headers.get("cid")
            else:
                cid = get_correlation_id()

            set_correlation_id(cid)
            # run other overide before_start
            return super().before_start(task_id, args, kwargs)

        def after_return(self, status, retval, task_id, args, kwargs, einfo):
            logger.debug(
                "task done",
                extra={"celery_task_id": task_id, "celery_task_retval": str(retval)},
            )
            return super().after_return(status, retval, task_id, args, kwargs, einfo)

        def on_failure(self, exc, task_id, args, kwargs, einfo):
            logger.critical(
                "task failed",
                extra={"celery_task_id": task_id, "celery_task_exc_info": str(einfo)},
            )
            return super().on_failure(exc, task_id, args, kwargs, einfo)

        def on_success(self, retval, task_id, args, kwargs):
            logger.debug(
                "task success",
                extra={"celery_task_id": task_id, "celery_task_retval": str(retval)},
            )
            return super().on_success(retval, task_id, args, kwargs)

    return CeleryCustomTask



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging
from typing import Any

from .context import get_correlation_id


class CorrelationIdFilter(logging.Filter):
    """Logging filter to attach request/response information to log records"""

    def __init__(self, name: str = "correlation_id"):
        super().__init__(name=name)

    def filter(self, record: logging.LogRecord) -> bool:
        """
        Attach request/response information to the log record.

        This includes correlation ID, request details, response details,
        timing information, and exception details if any.
        """
        # Basic correlation
        record.cid = get_correlation_id()
        return True


class EnsureValidExtraFilter(logging.Filter):
    def __init__(self, name="ensure_valid_extra", strict: bool = False):
        super().__init__(name)
        self.strict = strict
        self.by_pass_keys = {
            "name",
            "msg",
            "args",
            "levelname",
            "levelno",
            "pathname",
            "filename",
            "module",
            "exc_info",
            "exc_text",
            "stack_info",
            "lineno",
            "funcName",
            "created",
            "msecs",
            "relativeCreated",
            "thread",
            "threadName",
            "processName",
            "process",
            "message",
            "asctime",
        }

    def _validate_value(self, value: Any) -> Any:
        """Check kiểu dữ liệu hợp lệ."""
        if isinstance(value, (str, int, float)):
            return value
        elif isinstance(value, dict):
            if all(
                isinstance(k, str) and isinstance(v, (str, int, float))
                for k, v in value.items()
            ):
                return value
        elif isinstance(value, list):
            if all(isinstance(v, (str, int, float)) for v in value):
                return value

        if self.strict:
            raise TypeError(f"Invalid extra value type: {value!r} ({type(value)})")
        return str(value)  # fallback: convert sang string

    def filter(self, record: logging.LogRecord) -> bool:
        if hasattr(record, "__dict__"):
            for k, v in record.__dict__.items():
                if k in self.by_pass_keys:
                    continue
                if v is None or isinstance(v, (str, int, float)):
                    continue
                elif isinstance(v, dict):
                    if all(
                        isinstance(kk, str) and isinstance(vv, (str, int, float))
                        for kk, vv in v.items()
                    ):
                        continue
                    else:
                        record.__dict__[k] = str(v)
                elif isinstance(v, list):
                    if all(isinstance(vv, (str, int, float)) for vv in v):
                        continue
                    else:
                        record.__dict__[k] = str(v)
                else:
                    record.__dict__[k] = str(v)
        return True



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from datetime import datetime
from typing import Callable, List, Optional

from fastapi import FastAPI
from fastapi.responses import HTMLResponse
from socketio import ASGIApp

default_page = """
    <!DOCTYPE html>
    <html lang="en">
    <body>
        Hello, welcome to {project_name} {service_name}
    </body>
    </html>
"""


def create_fastapi_server_app(
    project_name: str,
    service_name: str,
    service_path: str,
    serivce_port: int,
    api_app: FastAPI,
    ws_enable: bool = False,
    ws_app: Optional[ASGIApp] = None,
    ws_event_handlers_initializer: Optional[Callable] = None,
    startup_events: List[Callable] = [],
    shutdown_events: List[Callable] = [],
):
    app = FastAPI(
        title=project_name,
        description=f"{project_name} {service_name}",
        on_startup=startup_events,
        on_shutdown=shutdown_events,
        docs_url=None,
    )

    # mount api
    app.mount(f"/{service_path}/api", api_app)

    # mount ws
    if ws_enable and ws_app and ws_event_handlers_initializer:
        app.mount(f"/{service_path}/ws", ws_app)
        ws_event_handlers_initializer()

    service_home_page = default_page.format(
        project_name=project_name, service_name=service_name
    )

    @app.get("/", response_class=HTMLResponse)
    async def root():
        return service_home_page

    @app.get(f"/{service_path}", response_class=HTMLResponse)
    async def root_service():
        return service_home_page

    @app.get("/server-info")
    async def traefik_heath_check():
        resp = {
            "status": "running",
            "info": {
                "name": service_name,
                "port": serivce_port,
                "code": "python-fastapi",
            },
            "date": datetime.utcnow().isoformat(timespec="milliseconds") + "Z",
            #  "2022-10-18T04:40:40.207Z"
        }

        return resp

    return app



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from .mongodb import new_umongo_instance
from .redis import RedisClient

__all__ = [
    "new_umongo_instance",
    "RedisClient",
]



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from .api_app import create_fastapi_api_app
from .api_server import create_fastapi_server_app

__all__ = [
    "create_fastapi_api_app",
    "create_fastapi_server_app",
]



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import asyncio
import logging
from typing import Any, Dict

import httpx

from ..abstractions import SingletonMeta


class BaseAsyncApiConnector(metaclass=SingletonMeta):
    def __init__(
        self, base_url: str, timeout: int = 10, ssl_verify: bool = False, **kwargs
    ):
        self._httpx_client: httpx.AsyncClient = None
        self._base_url = base_url
        self._timeout = timeout
        self._ssl_verify = ssl_verify
        self._logger = kwargs.get("logger", logging.getLogger(__name__))

    def _init_httpx_async_client(self):
        if not self._httpx_client or not isinstance(
            self._httpx_client, httpx.AsyncClient
        ):
            self._httpx_client = httpx.AsyncClient(
                base_url=self._base_url, timeout=self._timeout, verify=self._ssl_verify
            )

    def get_httpx_async_client(self) -> httpx.AsyncClient:
        self._init_httpx_async_client()
        return self._httpx_client

    def close(self):
        if self._httpx_client:
            asyncio.run(self._httpx_client.aclose())
            self._httpx_client = None

    async def _fetch(
        self,
        method: str,
        url: str,
        headers: Dict = {},
        params: Dict = {},
        payload: Any = None,
        files: Any = None,
        auth: Any = None,
        ssl_verify: bool = False,
        **kwargs
    ) -> httpx.Response:
        if not self._httpx_client:
            self._httpx_client = httpx.AsyncClient(
                base_url=self._base_url, timeout=self._timeout, verify=ssl_verify
            )

        response = await self._httpx_client.request(
            method=method,
            url=url,
            headers=headers,
            params=params,
            json=payload,
            files=files,
            auth=auth,
            **kwargs
        )
        response.raise_for_status()
        return response

    async def _fetch_get_json_or_none(
        self,
        url: str,
        headers: Dict = {},
        params: Dict = {},
        auth: Any = None,
        ssl_verify: bool = False,
        **kwargs
    ) -> Any:
        try:
            response = await self._fetch(
                method="GET",
                url=url,
                headers=headers,
                params=params,
                auth=auth,
                ssl_verify=ssl_verify,
                **kwargs
            )

            return response.json()
        except Exception as e:
            raise e



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging
import sys

from pythonjsonlogger.json import JsonFormatter


def create_json_logger(logger_name: str, level: int = logging.INFO) -> logging.Logger:
    """
    Create logger which write output to console and log file.
    Logging levels: https://docs.python.org/3/library/logging.html#logging-levels
    Format options: https://docs.python.org/3/library/logging.html#logrecord-attributes

    Args:
        logger_name (str): Name to identify logger and used as log file's name.
        level (int, optional): Level of log message will be included. Defaults to logging.INFO.

    Returns:
        Logger: Logger class for manipulate message
    """
    logger = logging.getLogger(logger_name)
    logger.level = level
    logger.propagate = False

    # Console handler
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setFormatter(JsonFormatter())
    logger.addHandler(console_handler)

    return logger



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging
import time


def monkey_patch_httpx(logger: logging.Logger):
    """Monkey patch httpx to log request/response information"""
    try:
        import httpx
    except ImportError:
        logger.warning("httpx is not installed, skipping httpx monkey patch")
        return

    def _wrap_sync_send(original_send):
        def new_send(self, request, *args, **kwargs):
            start_time = time.time_ns()
            has_exc = False
            try:
                response = original_send(self, request, *args, **kwargs)
                return response
            except Exception as e:
                has_exc = True
                logger.exception(
                    f"httpx sync request {request.url} failed: {e}",
                    extra={
                        "req_to_url": str(request.url),
                        "req_to_method": str(request.method),
                        "req_to_headers": str(dict(request.headers)),
                        "resp_from_status": -1,
                        "loc": "core0001",
                    },
                )
                raise
            finally:
                if not has_exc:
                    resp_status = response.status_code if "response" in locals() else 0
                    elapsed_ms = (time.time_ns() - start_time) / 1_000_000
                    logger.info(
                        f"httpx sync request {request.url} {resp_status} {elapsed_ms}ms",
                        extra={
                            "req_to_url": str(request.url),
                            "req_to_method": str(request.method),
                            "req_to_headers": str(dict(request.headers)),
                            "resp_from_status": resp_status,
                            "resp_from_headers": (
                                str(dict(response.headers))
                                if "response" in locals()
                                else ""
                            ),
                            "loc": "core0002",
                        },
                    )

        return new_send

    def _wrap_async_send(original_send):
        async def new_send(self, request, *args, **kwargs):
            start_time = time.time_ns()
            has_exc = False
            try:
                response = await original_send(self, request, *args, **kwargs)
                return response
            except Exception as e:
                has_exc = True
                logger.exception(
                    f"httpx async request {request.url} failed: {e}",
                    extra={
                        "req_to_url": str(request.url),
                        "req_to_method": str(request.method),
                        "req_to_headers": str(dict(request.headers)),
                        "resp_from_status": -1,
                        "loc": "core0003",
                    },
                )
                raise
            finally:
                if not has_exc:
                    resp_status = response.status_code if "response" in locals() else 0
                    elapsed_ms = (time.time_ns() - start_time) / 1_000_000
                    logger.info(
                        f"httpx async request {request.url} {resp_status} {elapsed_ms}ms",
                        extra={
                            "req_to_url": str(request.url),
                            "req_to_method": str(request.method),
                            "req_to_headers": str(dict(request.headers)),
                            "resp_from_status": resp_status,
                            "resp_from_headers": (
                                str(dict(response.headers))
                                if "response" in locals()
                                else ""
                            ),
                            "loc": "core0004",
                        },
                    )

        return new_send

    if not getattr(httpx, "_patched", False):
        httpx.Client.send = _wrap_sync_send(httpx.Client.send)
        httpx.AsyncClient.send = _wrap_async_send(httpx.AsyncClient.send)
        httpx._patched = True
        logger.info(
            "httpx has been monkey patched for logging", extra={"loc": "core0005"}
        )



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from typing import Optional

from pydantic_settings import BaseSettings


class RabbitMQSettings(BaseSettings):
    RBMQ_URL: Optional[str] = "amqp://username:password@localhost:5672,localhost:5673//"
    RBMQ_CONSUMER_PREFETCH_COUNT: Optional[int] = 20



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>


import asyncio

from motor.motor_asyncio import AsyncIOMotorClient
from umongo.frameworks import MotorAsyncIOInstance


def new_umongo_instance(mongo_uri: str, db_name: str):
    """
    Create an instance of mongo db to register model
    mongo-uri example:
        "mongodb://user:pass@host01:port01,host02:port02,host03:port03/DbName?replicaSet=rs0&authSource=admin",


    Args:
        mongo_uri (str): MongoDb URI for connection.
        db_name (str): Name of the database will be used.

    Returns:
        MotorAsyncIOInstance: implementation for motor-asyncio
    """

    motor_client = AsyncIOMotorClient(mongo_uri)[db_name]
    motor_client.get_io_loop = asyncio.get_running_loop
    instance = MotorAsyncIOInstance(motor_client)
    return instance



from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.http.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor


def setup_open_telemetry(
    endpoint: str = "http://otel-collector-hni01.ftel.scc/v1/traces",
    service_name: str = "EasyCore",
    name: str = "core",
    environment: str = "dev",
) -> None:
    # Set up OpenTelemetry resources
    resource = Resource(
        attributes={
            SERVICE_NAME: f"{service_name}-{name}",
            "service.environment": environment,
        }
    )

    # Configure trace provider and OTLP exporter
    provider = TracerProvider(resource=resource)
    otlp_exporter = OTLPSpanExporter(
        endpoint=endpoint,  # Default OTLP HTTP Collector endpoint
    )

    span_processor = BatchSpanProcessor(otlp_exporter)
    provider.add_span_processor(span_processor)

    # Set the global trace provider
    trace.set_tracer_provider(provider)
